#!/usr/bin/env python

"""

Slice out traces from a segy file

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: segyslice.py,v 1.4 2005/09/15 19:02:47 schwehr Exp $

http://www.python.org/doc/lib/module-optparse.html

http://schwehr.org/software/segy-py/
"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.4 $' # See man ident
__date__ = '$Date: 2005/09/15 19:02:47 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'
#__credits__ = """"""

BOMBASTIC= 4
VERBOSE  = 3
TRACE    = 2
TERSE    = 1
ALWAYS   = 0


import os
import sys

from segy import Segy
import segy

from optparse import OptionParser

vcvs = "$Revision: 1.4 $"
vcvs = vcvs[vcvs.find(':')+2:]
vcvs = vcvs[:vcvs.find('$')-1]
version_cvs = vcvs
del (vcvs) # stop polution

# Track the version of segy-py.  If I use setup.py, which one gets imported?
from segy_version import VERSION

myparser = OptionParser(usage="%prog [options] key1 key2 key3 ...",
                      version="%prog "+VERSION+' - '+version_cvs)

myparser.add_option('-d','--driver',dest="drv",
                    help="specify which driver to parse data file.  [default: %default]",
                    default="segy_drv_rev1")

myparser.add_option('-o','--out',dest="outfile",
                    help="Write segy to this file.  '.sgy' will be appended for you.  [default: %default]",
                    default="out")

myparser.add_option('-x','--ext',dest="ext",
                    help="Extension to append to outfile.  e.g. .xstar [default: %default]",
                    default=".sgy")

myparser.add_option('-i','--in',dest="infile",
                    help="Read segy to this file.  [default: %default]",
                    default="in.sgy")

myparser.add_option('-m','--min',dest='min',default=None, type='int',
                    help='Minimum trace to start writing [default: %default]')
myparser.add_option('-M','--max',dest='max',default=None, type='int',
                    help='Maximum trace to end writing on [default: %default]')

# Put a whole in the data
myparser.add_option('-e','--min-except',dest='minExcept',default=None, type='int',
                    help='Minimum trace to start skipping [default: %default]')
myparser.add_option('-E','--max-except',dest='maxExcept',default=None, type='int',
                    help='Maximum trace to end skipping [default: %default]')
# FIX: make sure that if -e then also -E


myparser.add_option('-k','--keys',dest='sliceOnKeys',action='store_true', default=False,
                    help='Slice the file anytime a key changes.'
                    +'  Specify keys anythere on the commandline.'
                    +'  Filenames will have the traceNumbers appended.')


myparser.add_option('-v','--verbose',action="count",dest='verbosity',default=0,
                    help='how much information to give.  Specify multiple times to increase verbosity')
myparser.add_option('--verbosity',dest='verbosity',type='int',
                    help='Specify verbosity.  Should be in the range of '
                    +str(ALWAYS)+'...'+str(BOMBASTIC)+' (None...Bombastic)')

######################################################################



if __name__ == '__main__':
    (options,args) = myparser.parse_args()
    keys = args
    
    if options.verbosity >= VERBOSE:   print 'Option values: ', options
    if options.verbosity >= BOMBASTIC: print 'Arg    values: ', args

    if ((options.minExcept != None and options.maxExcept == None) or
        (options.minExcept == None and options.maxExcept != None)):
        sys.exit("ERROR: must specify both -e and -E")

    if options.minExcept != None and options.minExcept > options.maxExcept:
        sys.exit("ERROR: min except must be <= max except")
        

    s = Segy(options.infile, options.drv, options.verbosity)

    
    numTraces = s.getNumberOfTraces()
    #print numTraces

    if options.min == None: options.min = 1 # FIX: start counting from 1?????
    if options.max == None: options.max = s.getNumberOfTraces() # FIX: off by one?
        
    traces= []
    if options.minExcept != None:
        # poke a hole
        traces = range(options.min, options.minExcept)
        traces += range(options.maxExcept+1,options.max)
    else:
        traces = range(options.min,options.max)


    if options.sliceOnKeys:
        if len(keys) == 0: sys.exit ("ERROR: must specify at least one key field.  e.g. SweepLength")
        traceStart = options.min
        filename = options.outfile+ ("-%04d" % traceStart) + options.ext
        print 'FILE: ' , filename
        out = open(filename,'w')
        curVals = {}
        for key in keys:
            val = s.getTraceHdr(traceStart).getHeader(key)
            print key,val
            curVals[key] = val
        if options.verbosity >= BOMBASTIC: print 'Initial values for keys:\n   ',curVals

        
        for traceNum in traces:
            if options.verbosity >= BOMBASTIC:
                if traceNum % 50 == 0: print traceNum

            foundNew = False

            for key in keys:
                val = s.getTraceHdr(traceNum).getHeader(key)
                if val != curVals[key]:
                    if options.verbosity>=TRACE: print "  Value change: ", val, ' -> ',curVals[key], ' for key ',key
                    foundNew = True
                    curVals[key]=val
            if foundNew:
                if options.verbosity>=TRACE: print 'closing file: ', filename
                out.close()
                # FIX: rename file to have start and ending tracenumber?
                filename = options.outfile+ ("-%04d" % traceNum) + options.ext
                if options.verbosity>=TRACE: print 'opening file: ', filename
                out = open(filename,'w')
                out.write(s.getHeader().getEntireRawHdr())

            out.write(s.getRawTrace(traceNum))


    else: ###### NO KEYS ######
        out = open(options.outfile+options.ext,'w')

        # Duplicate the text and binary headers
        hdr = s.getHeader().getEntireRawHdr()
        out.write(hdr)

        for traceNum in traces:
            if options.verbosity >= BOMBASTIC: print traceNum
            elif options.verbosity >= TRACE:
                if (traceNum%100)==0:
                    print traceNum

            d = s.getRawTrace(traceNum)

            out.write(d)
        
