#!/usr/bin/env python
"""

Dump or list a range of shots from a database in a nice table

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: segysqldump.py,v 1.5 2005/09/15 16:06:05 schwehr Exp $

"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.5 $' # See man ident
__date__ = '$Date: 2005/09/15 16:06:05 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'
__credits__ = """LSD inspired... the program, not the drug"""

import sqlite
import os
import sys

BOMBASTIC= 4
VERBOSE  = 3
TRACE    = 2
TERSE    = 1
ALWAYS   = 0


from optparse import OptionParser

vcvs = "$Revision: 1.5 $"
vcvs = vcvs[vcvs.find(':')+2:]
vcvs = vcvs[:vcvs.find('$')-1]
version_cvs = vcvs
del (vcvs) # stop polution

# Track the version of segy-py.  If I use setup.py, which one gets imported?
from segy_version import VERSION

myparser = OptionParser(usage="%prog [options]",
                      version="%prog "+VERSION+' - '+version_cvs)
myparser.add_option('-d','--database',dest='dbfilename',default='segy.db',
                    help='Database containing imported segy file and trace header info [default: %default]')

myparser.add_option('-n','--nth-trace',dest='nthTrace',type='int',default='500',
                    help='How many trace entries to skip before writing out a label [default: %default]')


myparser.add_option('-w','--width',type="int",dest='width',default=12,
                    help='Character width of each field [default: %default]')


myparser.add_option('-v',action="count",dest='verbosity',default=0,
                    help='how much information to give.  Specify multiple times to increase verbosity')
myparser.add_option('--verbosity',dest='verbosity',type='int',
                    help='Specify verbosity.  Should be in the range of '
                    +str(ALWAYS)+'...'+str(BOMBASTIC)+' (None...Bombastic)')

myparser.add_option('-u','--from-utm',action="store_true",dest='fromUTM',default=False,
                    help='Assume x and y are in UTM and convert to Long/Lat')
myparser.add_option('-z','--utm-zone',type="int",dest='utmZone',default=None,
                    help='Specify the UTM zone if converting from UTM to Long/Lat')
myparser.add_option('-c','--coord-unit',type="int",dest='forceCoordUnit',default=-1,
                    help='Force a SEGY coordinate system, e.g. 2 == seconds of arc.  Use this for short format databases. [default: follow file]')

myparser.add_option('-x','--flip-x',dest='flipX',default=False,action='store_true',
                    help='Multiply the x coordinate by -1')
myparser.add_option('-y','--flip-y',dest='flipY',default=False,action='store_true',
                    help='Multiply the x coordinate by -1')
myparser.add_option('-H','--force-hemisphere',type="int",dest='forceHemisphere',default=-1,
                    help='Force hemisphere for shots that have bad navigation.'+
                    '  0-NW,  1-NE,  2-SW,  3-SE')


from segysqlgmt import CoordTransform

######################################################################

def hasField(db_cx, fieldName, table):
    """Return true if a field exists in a table
    FIX: the dependency on the spaces before the fieldName is dangerous
    """

    cu = cx.cursor()
    cu.execute('select sql from sqlite_master where name = "'+table+'"')
    row = cu.fetchone()
    if -1 == row['sql'].find(' '+fieldName+' '): return False
    return True

######################################################################
if __name__ == "__main__":
    (options,args) = myparser.parse_args()

    if options.verbosity >= TRACE:
        print "Verbosity = ", options.verbosity

    cx = sqlite.connect(options.dbfilename)

    ct = None
    if options.fromUTM:
        if options.verbosity >= TRACE:
            print "Setting up UTM to LonLat conversions for zone: ", options.utmZone
        ct = CoordTransform(cx,options)

    cu = cx.cursor()
       

    fields = []
    fields.append('fileKey')
    fields.append('traceNumber')
    fields.append('X')
    fields.append('Y')
    #fields.append('GrpX')
    #fields.append('GrpY')
    fields.append('EnsembleNo')
    fields.append('FieldRecNo')
    fields.append('Xensemble')
    fields.append('Yensemble')
    fields.append('Year')
    fields.append('Day')
    fields.append('Hour')
    fields.append('Min')
    fields.append('Shotpoint')
    fields.append('TraceNoEnsemble')

    # All extra args are fields to print
    for field in args:
        fields.append(field)

    fieldsShort = []
    for field in fields:
        if hasField(cx, field, 'segyTrace'):
            fieldsShort.append(field)
            if options.verbosity>=BOMBASTIC: print 'yes, has '+field
        else:
            if options.verbosity>=TRACE: print ' NO! Missing '+field

    print fields
    print fieldsShort
    
    fieldStr = fieldsShort[0]
    for i in fieldsShort[1:]:
        fieldStr += ','+i

    fields = fieldsShort
    
    searchStr  = "SELECT "+fieldStr
    searchStr += " FROM segyTrace"
    searchStr += " WHERE traceNumber % "+str(options.nthTrace)+"= 0"
    #if (None != options.minFileKey): searchStr += ' fileKey>='+str(options.minFileKey)
    #if (None != options.minFileKey) and (None != options.maxFileKey): searchStr += ' and'
    #if (None != options.maxFileKey): searchStr += ' fileKey<='+str(options.maxFileKey)
    searchStr += ';'

    if options.verbosity>=BOMBASTIC: print searchStr

    width = options.width
    cu.execute(searchStr)
    count = 0;
    for row in cu.fetchall():
        if count%50 == 0:
            print
            for field in fields:
                while len(field)<width: field += ' '
                while len(field)>width: field = field[:-1]
                print field,' ',
            print '\n'
        count += 1
        x = row['X']
        y = row['Y']
        if options.fromUTM:
            x,y = ct.getLonLat(x,y)
        for field in fields:

            val = str(row[field])
            if options.fromUTM:
                if 'X' == field: val = str(x)
                if 'Y' == field: val = str(y)

            while len(val)<width: val += ' '
            if len(val)>width: val = val[:width]
            #if len(val)>width: val = val[2:]
            print val,' ',

        print
